@@ -33,12 +33,14 @@ from bzrlib.plugins.qbzr.lib.util import (
     QBzrWindow,
     ToolBarThrobberWidget,
     get_apparent_author_name,
-    get_set_encoding,
-    runs_in_loading_queue,
     get_icon,
-    FindToolbar,
     get_monospace_font,
+    get_set_encoding,
+    get_set_tab_width_chars,
+    get_tab_width_pixels,
+    runs_in_loading_queue,
     )
+from bzrlib.plugins.qbzr.lib.widgets.toolbars import FindToolbar
 from bzrlib.plugins.qbzr.lib.uifactory import ui_current_widget
 from bzrlib.plugins.qbzr.lib.trace import reports_exception
 from bzrlib.plugins.qbzr.lib.logwidget import LogList
@@ -52,6 +54,7 @@ from bzrlib.workingtree import WorkingTree
 from bzrlib.revisiontree import RevisionTree
 from bzrlib.plugins.qbzr.lib.revisionmessagebrowser import LogListRevisionMessageBrowser
 from bzrlib.plugins.qbzr.lib.encoding_selector import EncodingMenuSelector
+from bzrlib.plugins.qbzr.lib.widgets.tab_width_selector import TabWidthMenuSelector
 from bzrlib.plugins.qbzr.lib.syntaxhighlighter import highlight_document
 from bzrlib.plugins.qbzr.lib.revtreeview import paint_revno, get_text_color
 from bzrlib.plugins.qbzr.lib import logmodel
@@ -334,7 +337,15 @@ class AnnotateWindow(QBzrWindow):
         self.connect(word_wrap,
                      QtCore.SIGNAL("toggled (bool)"),
                      self.word_wrap_toggle)
+
+        def setTabStopWidth(tw):
+            self.text_edit.setTabStopWidth(get_tab_width_pixels(tab_width_chars=tw))
+            get_set_tab_width_chars(branch=self.branch,tab_width_chars=tw)
+        self.tab_width_selector = TabWidthMenuSelector(get_set_tab_width_chars(branch=branch),
+                gettext("Tab Width"),
+                setTabStopWidth)
         
+        view_menu.addMenu(self.tab_width_selector)
         view_menu.addMenu(self.encoding_selector)
         view_menu.addAction(word_wrap)
         
