@@ -33,6 +33,7 @@ from bzrlib.plugins.qbzr.lib.util import (
     )
 from bzrlib.plugins.qbzr.lib.trace import reports_exception, SUB_LOAD_METHOD
 from bzrlib.plugins.qbzr.lib.uifactory import ui_current_widget
+from bzrlib.plugins.qbzr.lib.loggraphviz import GhostRevisionError
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), '''
@@ -571,6 +572,10 @@ class FileListContainer(QtGui.QWidget):
         self.file_list_context_menu_cat = \
             self.file_list_context_menu.addAction(gettext("View file"),
                                                   self.show_file_content)
+        self.file_list_context_menu_save_old_file = \
+            self.file_list_context_menu.addAction(
+                            gettext("Save file on this revision as..."),
+                            self.save_old_revision_of_file)
         self.file_list_context_menu_revert_file = \
                 self.file_list_context_menu.addAction(
                                         gettext("Revert to this revision"),
@@ -626,51 +631,58 @@ class FileListContainer(QtGui.QWidget):
         
         if revids not in self.delta_cache:
             self.throbber.show()
-            repos = [gv.get_revid_branch(revid).repository for revid in revids]
-            if (repos[0].__class__.__name__ == 'SvnRepository' or
-                repos[1].__class__.__name__ == 'SvnRepository'):
-                # Loading trees from a remote svn repo is unusably slow.
-                # See https://bugs.launchpad.net/qbzr/+bug/450225
-                # If only 1 revision is selected, use a optimized svn method
-                # which actualy gets the server to do the delta,
-                # else, don't do any delta.
-                if count == 1:
-                    delta = repos[0].get_revision_delta(revids[0])
-                else:
-                    delta = None
+            try:
+                repos = [gv.get_revid_branch(revid).repository for revid in
+                                                                        revids]
+            except GhostRevisionError:
+                delta = None
             else:
-                if (len(repos)==2 and
-                    repos[0].base == repos[1].base):
-                    # Both revids are from the same repository. Load together.
-                    repos_revids = [(repos[0], revids)]
+                if (repos[0].__class__.__name__ == 'SvnRepository' or
+                    repos[1].__class__.__name__ == 'SvnRepository'):
+                    # Loading trees from a remote svn repo is unusably slow.
+                    # See https://bugs.launchpad.net/qbzr/+bug/450225
+                    # If only 1 revision is selected, use a optimized svn method
+                    # which actualy gets the server to do the delta,
+                    # else, don't do any delta.
+                    if count == 1:
+                        delta = repos[0].get_revision_delta(revids[0])
+                    else:
+                        delta = None
                 else:
-                    repos_revids = [(repo, [revid])
-                            for revid, repo in zip(revids, repos)]
-                
-                for repo, repo_revids in repos_revids:
-                    repo_revids = [revid for revid in repo_revids 
-                                   if revid not in self.tree_cache]
-                    if repo_revids:
-                        repo.lock_read()
-                        self.processEvents()
-                        try:
-                            for revid in repo_revids:
-                                if (revid.startswith(CURRENT_REVISION) and
-                                    gv_is_wtgv):
-                                    tree = gv.working_trees[revid]
-                                else:
-                                    tree = repo.revision_tree(revid)
-                                self.tree_cache[revid] = tree
+                    if (len(repos)==2 and
+                        repos[0].base == repos[1].base):
+                        # Both revids are from the same repository. Load
+                        #together.
+                        repos_revids = [(repos[0], revids)]
+                    else:
+                        repos_revids = [(repo, [revid])
+                                for revid, repo in zip(revids, repos)]
+                    
+                    for repo, repo_revids in repos_revids:
+                        repo_revids = [revid for revid in repo_revids 
+                                    if revid not in self.tree_cache]
+                        if repo_revids:
+                            repo.lock_read()
                             self.processEvents()
-                        finally:
-                            repo.unlock()
-                    self.processEvents()
-                
-                delta = self.tree_cache[revids[0]].changes_from(
-                                                self.tree_cache[revids[1]])
-            self.delta_cache[revids] = delta
-            self.throbber.hide()
-            self.processEvents()
+                            try:
+                                for revid in repo_revids:
+                                    if (revid.startswith(CURRENT_REVISION) and
+                                        gv_is_wtgv):
+                                        tree = gv.working_trees[revid]
+                                    else:
+                                        tree = repo.revision_tree(revid)
+                                    self.tree_cache[revid] = tree
+                                self.processEvents()
+                            finally:
+                                repo.unlock()
+                        self.processEvents()
+                    
+                    delta = self.tree_cache[revids[0]].changes_from(
+                                                    self.tree_cache[revids[1]])
+                self.delta_cache[revids] = delta
+            finally:
+                self.throbber.hide()
+                self.processEvents()
         else:
             delta = self.delta_cache[revids]
         
@@ -736,8 +748,8 @@ class FileListContainer(QtGui.QWidget):
         is_single_file = len(paths) == 1
         self.file_list_context_menu_annotate.setEnabled(is_single_file)
         self.file_list_context_menu_cat.setEnabled(is_single_file)
-        
-                   
+        self.file_list_context_menu_save_old_file.setEnabled(is_single_file)
+
         gv = self.log_list.log_model.graph_viz
         # It would be nice if there were more than one branch, that we
         # show a menu so the user can chose which branch actions should take
@@ -747,11 +759,10 @@ class FileListContainer(QtGui.QWidget):
 
         self.file_list_context_menu_revert_file.setEnabled(one_branch_with_tree)
         self.file_list_context_menu_revert_file.setVisible(one_branch_with_tree)
-            
-        
+
         self.file_list_context_menu.popup(
             self.file_list.viewport().mapToGlobal(pos))
-    
+
     def get_file_selection_indexes(self, index=None):
         if index is None:
             return self.file_list.selectionModel().selectedRows(0)
@@ -798,6 +809,37 @@ class FileListContainer(QtGui.QWidget):
         window.show()
         self.window().windows.append(window)
 
+    @ui_current_widget
+    def save_old_revision_of_file(self):
+        """Saves the selected file in its revision to a directory."""
+        paths, file_ids = self.get_file_selection_paths_and_ids()
+        (top_revid, old_revid), count = \
+            self.log_list.get_selection_top_and_parent_revids_and_count()
+
+        branch = self.log_list.log_model.graph_viz.get_revid_branch(top_revid)
+        tree = branch.repository.revision_tree(top_revid)
+        file_id = file_ids[0]
+        tree.lock_read()
+        try:
+            kind = tree.kind(file_id)
+            if kind == 'file':
+                file_content_bytes = tree.get_file_text(file_id)
+        finally:
+            tree.unlock()
+        if kind != 'file':
+            QtGui.QMessageBox.information(self, gettext("Not a file"),
+                gettext("Operation is supported for a single file only,\n"
+                        "not for a %s." % kind))
+            return
+        filename = QtGui.QFileDialog.getSaveFileName(
+                self, gettext("Save file in this revision as..."))
+        if filename:
+            f = open(unicode(filename), 'wb')
+            try:
+                f.write(file_content_bytes)
+            finally:
+                f.close()
+
     @ui_current_widget    
     def revert_file(self):
         """Reverts the file to what it was at the selected revision."""
